@@ -193,13 +193,13 @@ def combine_params(*param_dicts):
     Examples
     --------
     >>> CMRparams = {'short_name': 'ATL06', 'version': '002', 'temporal': '2019-02-20T00:00:00Z,2019-02-28T23:59:59Z', 'bounding_box': '-55,68,-48,71'}
-    >>> reqparams = {'page_size': 10, 'page_num': 1}
-    >>> icepyx.core.APIformatting.combine_params(CMRparams, reqparams)
+    >>> reqparams = {'page_size': 2000, 'page_num': 1}
+    >>> ipx.core.APIformatting.combine_params(CMRparams, reqparams)
     {'short_name': 'ATL06',
     'version': '002',
     'temporal': '2019-02-20T00:00:00Z,2019-02-28T23:59:59Z',
     'bounding_box': '-55,68,-48,71',
-    'page_size': 10,
+    'page_size': 2000,
     'page_num': 1}
     """
     params = {}
@@ -223,10 +223,10 @@ def to_string(params):
     Examples
     --------
     >>> CMRparams = {'short_name': 'ATL06', 'version': '002', 'temporal': '2019-02-20T00:00:00Z,2019-02-28T23:59:59Z', 'bounding_box': '-55,68,-48,71'}
-    >>> reqparams = {'page_size': 10, 'page_num': 1}
-    >>> params = icepyx.core.APIformatting.combine_params(CMRparams, reqparams)
-    >>> icepyx.core.APIformatting.to_string(params)
-    'short_name=ATL06&version=002&temporal=2019-02-20T00:00:00Z,2019-02-28T23:59:59Z&bounding_box=-55,68,-48,71&page_size=10&page_num=1'
+    >>> reqparams = {'page_size': 2000, 'page_num': 1}
+    >>> params = ipx.core.APIformatting.combine_params(CMRparams, reqparams)
+    >>> ipx.core.APIformatting.to_string(params)
+    'short_name=ATL06&version=002&temporal=2019-02-20T00:00:00Z,2019-02-28T23:59:59Z&bounding_box=-55,68,-48,71&page_size=2000&page_num=1'
     """
     param_list = []
     for k, v in params.items():
@@ -325,7 +325,7 @@ class Parameters:
             }
         elif self.partype == "required":
             self._poss_keys = {
-                "search": ["page_size", "page_num"],
+                "search": ["page_size"],
                 "download": [
                     "page_size",
                     "page_num",
@@ -441,8 +441,8 @@ class Parameters:
             else:
                 reqkeys = self.poss_keys[self._reqtype]
                 defaults = {
-                    "page_size": 10,
-                    "page_num": 1,
+                    "page_size": 2000,
+                    "page_num": 0,
                     "request_mode": "async",
                     "include_meta": "Y",
                     "client_string": "icepyx",
@@ -450,20 +450,11 @@ class Parameters:
                 for key in reqkeys:
                     if key in kwargs:
                         self._fmted_keys.update({key: kwargs[key]})
-                    #                 elif key in defaults:
-                    #                     if key is 'page_num':
-                    #                         pnum = math.ceil(len(is2obj.granules)/reqparams['page_size'])
-                    #                         if pnum > 0:
-                    #                             reqparams.update({key:pnum})
-                    #                         else:
-                    #                             reqparams.update({key:defaults[key]})
                     elif key in defaults:
                         self._fmted_keys.update({key: defaults[key]})
                     else:
                         pass
 
-                self._fmted_keys["page_num"] = 1
-
         else:
             if self.check_values == True and kwargs == None:
                 pass
