@@ -7,6 +7,7 @@ import json
 import numpy as np
 import os
 import pprint
+import warnings
 from xml.etree import ElementTree as ET
 import zipfile
 
@@ -34,7 +35,7 @@ def info(grans):
 
 # DevNote: currently this fn is not tested
 # DevNote: could add flag to separate ascending and descending orbits based on ATL03 granule region
-def gran_IDs(grans, ids=True, cycles=False, tracks=False, dates=False):
+def gran_IDs(grans, ids=True, cycles=False, tracks=False, dates=False, s3urls=False):
     """
     Returns a list of granule information for each granule dictionary in the input list of granule dictionaries.
     Granule info may be from a list of those available from NSIDC (for ordering/download)
@@ -51,7 +52,12 @@ def gran_IDs(grans, ids=True, cycles=False, tracks=False, dates=False):
     tracks : boolean, default False
         Return a list of the available Reference Ground Tracks (RGTs) for the granule dictionary
     dates : boolean, default False
-        Return a list of the available dates for the list of granual dictionaries.
+        Return a list of the available dates for the granule dictionary.
+    s3urls : boolean, default False
+        Return a a list of AWS s3 urls for the available granules in the granule dictionary.
+        Note: currently, NSIDC does not provide metadata on which granules are available on s3.
+        Thus, all of the urls may not be valid and may return FileNotFoundErrors.
+        s3 data access is currently limited access to beta testers.
     """
     assert len(grans) > 0, "Your data object has no granules associated with it"
     # regular expression for extracting parameters from file names
@@ -63,41 +69,52 @@ def gran_IDs(grans, ids=True, cycles=False, tracks=False, dates=False):
     gran_cycles = []
     gran_tracks = []
     gran_dates = []
+    gran_s3urls = []
     for gran in grans:
         producer_granule_id = gran["producer_granule_id"]
         gran_ids.append(producer_granule_id)
-        # PRD: ICESat-2 product
-        # HEM: Sea Ice Hemisphere flag
-        # YY,MM,DD,HH,MN,SS: Year, Month, Day, Hour, Minute, Second
-        # TRK: Reference Ground Track (RGT)
-        # CYCL: Orbital Cycle
-        # GRAN: Granule region (1-14)
-        # RL: Data Release
-        # VERS: Product Version
-        # AUX: Auxiliary flags
-        # SFX: Suffix (h5)
-        (
-            PRD,
-            HEM,
-            YY,
-            MM,
-            DD,
-            HH,
-            MN,
-            SS,
-            TRK,
-            CYCL,
-            GRAN,
-            RL,
-            VERS,
-            AUX,
-            SFX,
-        ) = rx.findall(producer_granule_id).pop()
-        gran_cycles.append(CYCL)
-        gran_tracks.append(TRK)
-        gran_dates.append(
-            str(datetime.datetime(year=int(YY), month=int(MM), day=int(DD)).date())
-        )
+
+        if int(gran["producer_granule_id"][3:5]) > 13:
+            continue
+            # ultimately use this to get the s3urls from the metadata
+            # gran_s3urls.append(gran["links"][])
+
+        else:
+            # PRD: ICESat-2 product
+            # HEM: Sea Ice Hemisphere flag
+            # YY,MM,DD,HH,MN,SS: Year, Month, Day, Hour, Minute, Second
+            # TRK: Reference Ground Track (RGT)
+            # CYCL: Orbital Cycle
+            # GRAN: Granule region (1-14)
+            # RL: Data Release
+            # VERS: Product Version
+            # AUX: Auxiliary flags
+            # SFX: Suffix (h5)
+            (
+                PRD,
+                HEM,
+                YY,
+                MM,
+                DD,
+                HH,
+                MN,
+                SS,
+                TRK,
+                CYCL,
+                GRAN,
+                RL,
+                VERS,
+                AUX,
+                SFX,
+            ) = rx.findall(producer_granule_id).pop()
+            gran_cycles.append(CYCL)
+            gran_tracks.append(TRK)
+            gran_dates.append(
+                str(datetime.datetime(year=int(YY), month=int(MM), day=int(DD)).date())
+            )
+            gran_s3urls.append(
+                f"s3://nsidc-cumulus-prod-protected/ATLAS/{PRD}/{RL}/{YY}/{MM}/{DD}/{producer_granule_id}"
+            )
     # list of granule parameters
     gran_list = []
     # granule IDs
@@ -112,6 +129,14 @@ def gran_IDs(grans, ids=True, cycles=False, tracks=False, dates=False):
     # granule date
     if dates:
         gran_list.append(gran_dates)
+    # AWS s3 url
+    if s3urls:
+        warnings.filterwarnings("always")
+        warnings.warn(
+            "You MUST be pre-authenticated by NSIDC as a beta tester to have cloud access to ICESat-2 data",
+            UserWarning,
+        )
+        gran_list.append(gran_s3urls)
     # return the list of granule parameters
     return gran_list
 
@@ -181,16 +206,28 @@ class Granules:
 
         headers = {"Accept": "application/json", "Client-Id": "icepyx"}
         # note we should also check for errors whenever we ping NSIDC-API - make a function to check for errors
+
+        params = apifmt.combine_params(
+            CMRparams, {k: reqparams[k] for k in ["page_size"]}
+        )
+
+        cmr_search_after = None
+
         while True:
-            params = apifmt.combine_params(
-                CMRparams, {k: reqparams[k] for k in ("page_size", "page_num")}
-            )
+            if cmr_search_after is not None:
+                headers["CMR-Search-After"] = cmr_search_after
+
             response = requests.get(
                 granule_search_url,
                 headers=headers,
                 params=apifmt.to_string(params),
             )
 
+            try:
+                cmr_search_after = response.headers["CMR-Search-After"]
+            except KeyError:
+                cmr_search_after = None
+
             try:
                 response.raise_for_status()
             except requests.HTTPError as e:
@@ -205,16 +242,13 @@ class Granules:
 
             results = json.loads(response.content)
             if not results["feed"]["entry"]:
-                # Out of results, so break out of loop
+                assert len(self.avail) == int(
+                    response.headers["CMR-Hits"]
+                ), "Search failure - unexpected number of results"
                 break
 
-            # Collect results and increment page_num
+            # Collect results
             self.avail.extend(results["feed"]["entry"])
-            reqparams["page_num"] += 1
-
-        # DevNote: The above calculated page_num is wrong when mod(granule number, page_size)=0.
-        # print(reqparams['page_num'])
-        reqparams["page_num"] = int(np.ceil(len(self.avail) / reqparams["page_size"]))
 
         assert (
             len(self.avail) > 0
@@ -280,11 +314,8 @@ class Granules:
             )
 
         base_url = "https://n5eil02u.ecs.nsidc.org/egi/request"
-        # DevGoal: get the base_url from the granules?
 
-        self.get_avail(
-            CMRparams, reqparams
-        )  # this way the reqparams['page_num'] is updated
+        self.get_avail(CMRparams, reqparams)
 
         if subset is False:
             request_params = apifmt.combine_params(
@@ -295,27 +326,30 @@ class Granules:
 
         order_fn = ".order_restart"
 
+        total_pages = int(np.ceil(len(self.avail) / reqparams["page_size"]))
         print(
             "Total number of data order requests is ",
-            request_params["page_num"],
+            total_pages,
             " for ",
             len(self.avail),
             " granules.",
         )
-        # DevNote/05/27/20/: Their page_num values are the same, but use the combined version anyway.
-        # I'm switching back to reqparams, because that value is not changed by the for loop. I shouldn't cause an issue either way, but I've had issues with mutable types in for loops elsewhere.
-        for i in range(reqparams["page_num"]):
-            #         for i in range(request_params['page_num']):
-            page_val = i + 1
 
+        if reqparams["page_num"] > 0:
+            pagenums = [reqparams["page_num"]]
+        else:
+            pagenums = range(1, total_pages + 1)
+
+        for page_num in pagenums:
             print(
                 "Data request ",
-                page_val,
+                page_num,
                 " of ",
-                reqparams["page_num"],
+                total_pages,
                 " is submitting to NSIDC",
             )
-            request_params.update({"page_num": page_val})
+            request_params = apifmt.combine_params(CMRparams, reqparams, subsetparams)
+            request_params.update({"page_num": page_num})
 
             # DevNote: earlier versions of the code used a file upload+post rather than putting the geometries
             # into the parameter dictionaries. However, this wasn't working with shapefiles, but this more general
